% This is an ad-hoc script to read in EEG/MEG files from the Mattarello MEG
% to fixing channel information (labels, locations, scales)

% import MNE constants
global FIFF;
if isempty(FIFF)
   FIFF = fiff_define_constants();
end

% get types of each channel, in channel ordering
channelTypes = chantype(EEG.chaninfo.fiffheader);

% standard label equivalents
M = containers.Map({'EEG002', 'EEG053', 'EEG057', 'EEG065', 'EEG121'}, {'FPz', 'T7', 'Cz', 'T8', 'Oz'});

% storage for coordinates
deviceCoordinates = zeros(EEG.nbchan,3); % MEG electrodes are in this system
headCoordinates = zeros(EEG.nbchan,3); % EEGLAB likes this system

% find first EEG channel in digitized sequence
% this finds the index of the reference channel - others will follow it
for d=1:size(EEG.chaninfo.fiffheader.orig.dig,2)
    if EEG.chaninfo.fiffheader.orig.dig(d).kind == FIFF.FIFFV_POINT_EEG % EEG electrode
        break
    end
end

% transformation from Polyhemus digitisation to MEG device coordinates
dig2deviceTransform = inv(EEG.chaninfo.fiffheader.orig.dev_head_t.trans);

% go through channels, and get device coordinates
for c=1:size(EEG.chaninfo.fiffheader.orig.chs,2)
    switch char(channelTypes(c)) % could also have checked if EEG.chaninfo.fiffheader.orig.chs.kind == FIFF.FIFFV_MEG_CH or FIFF.FIFFV_EEG_CH
        % if it EEG, transform and store
        case 'eeg'
            % temp fix for bipolar channels EEG061-4
            if str2num(EEG.chaninfo.fiffheader.orig.chs(c).ch_name(4:end)) >= 61 & str2num(EEG.chaninfo.fiffheader.orig.chs(c).ch_name(4:end)) <= 64
                disp(['skipping channel ' num2str(c) ' ' EEG.chaninfo.fiffheader.orig.chs(400).ch_name])
                continue
            end               
            d = d + 1;
            digCoordinates = double(EEG.chaninfo.fiffheader.orig.dig(d).r);
            transposedCoordinates = dig2deviceTransform * [digCoordinates; 1];
            deviceCoordinates(c,:) = transposedCoordinates(1:3)';
        % if it MEG, store
        case {'megmag', 'megplanar'}
            deviceCoordinates(c,:) = [double(EEG.chaninfo.fiffheader.orig.chs(c).loc(1:3))];
        % otherwise (e.g. trigger, physio channels) do nothing
        otherwise
            deviceCoordinates(c,:) = [NaN NaN NaN];
    end

    % go through labels and find the EEGLAB set indices of the landmark
    % points we're interested in for establishing new axes in head
    % coordinates
    if strcmpi(EEG.chaninfo.fiffheader.orig.ch_names(c),'EEG057')
        Cz=deviceCoordinates(c,:);
    elseif  strcmpi(EEG.chaninfo.fiffheader.orig.ch_names(c),'EEG002')
        FPz=deviceCoordinates(c,:);
    elseif  strcmpi(EEG.chaninfo.fiffheader.orig.ch_names(c),'EEG121')
        Oz=deviceCoordinates(c,:);
    elseif  strcmpi(EEG.chaninfo.fiffheader.orig.ch_names(c),'EEG053')
        T7=deviceCoordinates(c,:);
    elseif  strcmpi(EEG.chaninfo.fiffheader.orig.ch_names(c),'EEG065')
        T8=deviceCoordinates(c,:);
    end
end

% estimate the transformation from device to (EEGlab standard orientation) 
% "head coordinates" (x axis through Oz-FPz towards nose, y axis T8-T7 to 
% left, z axis upwards through Cz)
% (this code relies very heavily on Robert Oosterveld's headcoordinates.m,
% but that program required a different subset of points than I had)
% option 1: if have FPz, Oz, T7, T8
if exist('FPz') && exist('Oz') && exist('T7') && exist('T8')
    dirx = (FPz-Oz)/norm(FPz-Oz); % unit vector of FPz-Oz
    diry = (T7-T8)/norm(T7-T8); % ... same of T7-T8
    dirz = cross(dirx,diry); % ... perpendicular to these two planes
    origin = (FPz+Oz+T7+T8)/4; % ... midpoint of these four
% option 2: if have FPz, Oz and Cz
elseif exist('FPz') && exist('Oz') && exist('Cz')
    dirx = (FPz-Oz)/norm(FPz-Oz); % unit vector of FPz-Oz
    origin = (FPz+Oz)/2; % ... midpoint of these two
    dirz = (Cz-origin)/norm(Cz-origin); % unit vector Cz-origin
    diry = -cross(dirx,dirz); % ... perpendicular to these two planes
end
% compute the rotation matrix (again, all this copied from
% headcoordinates.m)
rot = eye(4);
rot(1:3,1:3) = inv(eye(3) / [dirx; diry; dirz]);
% compute the translation matrix
tra = eye(4);
tra(1:4,4)   = [-origin(:); 1];
% compute the full homogenous transformation matrix from these two
device2headTransform = rot * tra;

% transform all the coordinates
headCoordinates = [device2headTransform*[deviceCoordinates ones(length(deviceCoordinates),1)]']';
% go through channels and store in EEG structure 
for c=1:size(EEG.chanlocs,1)
    EEG.chanlocs(c).type = char(channelTypes(c));
    % should be something about scale here
    switch char(channelTypes(c))
        case {'eeg', 'megmag', 'megplanar'}
            EEG.chanlocs(c).X = headCoordinates(c,1);
            EEG.chanlocs(c).Y = headCoordinates(c,2);
            EEG.chanlocs(c).Z = headCoordinates(c,3);
    end
end
% fill in the variables for the other coordinate sets (spherical, polar)
% based on these cartesian ones
EEG=pop_chanedit(EEG,  'convert',{ 'cart2all'});
% this conversion produces an error saying that the sphere of channels
% hasn't been centred - but this isn't a good idea, since if you do centre,
% you should do it on the basis of the EEG ones only, but then only they
% are centred and then MEG and EEG locations are out of whack
% It ALSO wipes the fiffheader info from chaninfo

% TO DO
% could delete fiffheader now (EEG.chaninfo.fiffheader), but keep for now
% would be nice to have some interactive way to do dimensions - e.g. ask
%   for two channels for each axis... 
